import 'package:contentsquare_platform_interface/contentsquare_platform_interface.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

void main() {
  group('$ContentsquarePlatform', () {
    late ExtendsContentsquarePlatform contentsquarePlatform;

    setUp(() {
      contentsquarePlatform = ExtendsContentsquarePlatform();
    });

    test('Cannot be implemented with `implements`', () {
      expect(
        () {
          ContentsquarePlatform.instance = ImplementsContentsquarePlatform();
        },
        throwsAssertionError,
      );
    });

    test('Can be extended', () {
      ContentsquarePlatform.instance = ExtendsContentsquarePlatform();
    });

    test('Can be mocked with `implements`', () {
      ContentsquarePlatform.instance = MockContentsquarePlatform();
    });

    test('Default implementation of send should throw unimplemented error', () {
      expect(
        () => contentsquarePlatform.send('test'),
        throwsUnimplementedError,
      );
    });

    test(
        'Default implementation of sendTransaction should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.sendTransaction(42, 'test'),
        throwsUnimplementedError,
      );
    });

    test(
        'Default implementation of sendDynamicVar should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.sendDynamicVar('key'),
        throwsUnimplementedError,
      );
    });

    test('Default implementation of optIn should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.optIn(),
        throwsUnimplementedError,
      );
    });

    test('Default implementation of optOut should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.optOut(),
        throwsUnimplementedError,
      );
    });

    test(
        'Default implementation of completePurchase should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.forgetMe(),
        throwsUnimplementedError,
      );
    });

    test('Default implementation of getUserId should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.getUserId(),
        throwsUnimplementedError,
      );
    });

    test(
        'Default implementation of stopTracking should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.stopTracking(),
        throwsUnimplementedError,
      );
    });

    test(
        'Default implementation of resumeTracking should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.resumeTracking(),
        throwsUnimplementedError,
      );
    });

    test(
        'Default implementation of sendTouchEvent should throw unimplemented error',
        () {
      expect(
        () => contentsquarePlatform.sendTouchEvent(
            version: 1,
            type: ContentsquareGesture.tap,
            data: const TouchEventData(path: 'paht/to/my/widget')),
        throwsUnimplementedError,
      );
    });
  });
}

class ImplementsContentsquarePlatform implements ContentsquarePlatform {
  @override
  dynamic noSuchMethod(Invocation invocation) => super.noSuchMethod(invocation);
}

class MockContentsquarePlatform extends Mock
    with
        // ignore: prefer_mixin
        MockPlatformInterfaceMixin
    implements
        ContentsquarePlatform {}

class ExtendsContentsquarePlatform extends ContentsquarePlatform {}
